#include <stdint.h>
#include "system.h"
#include "def_prog.h"
#include "device\bell.h"
#include "device\lcd\lcd_hd44780.h"
#include "device\reg_io.h"
#include "module\display.h"

enum {_250MS = 250, _1000MS = 1000};

extern volatile uint8_t Flags_0;                          //  
extern volatile uint8_t Key;                              //   
extern volatile uint8_t DisplayBlink[];                   //     
extern volatile uint8_t *DisplayOut;                      //        LCD
extern volatile uint8_t RegIn_1;                          //  1  
extern volatile uint8_t RegIn_2;                          //  2  
extern volatile uint8_t RegOut_1;                         //  1  
extern volatile uint8_t RegOut_2;                         //  2  

extern const uint8_t __flash LCD_TableCharDecode[];
extern const uint8_t __flash LCD_TableCfg[];

static uint16_t CountInt;
static uint16_t KeyDelay;
static uint8_t  DisplayCount;
static uint8_t  BeepCount;
static uint8_t  RegInBit;
static uint16_t Recount_1s = _1000MS;
static uint8_t  Recount_250ms = _250MS;
static uint16_t BlinkCount = BLINK_COUNT;                 //    
static uint8_t  RegIn_1Count[8];
static uint8_t  RegIn_2Count[8];
static uint8_t  RegKeyCount;

//    
static const uint8_t __flash TableKey[] = { 0xFF,0x00,0x01,0xFF,0x02,0xFF,0xFF,0xFF,0x03,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF };

//==============================================================================
//    
//==============================================================================
#pragma inline = forced
static inline void filterRegIn(uint8_t bit, uint8_t reg_in, volatile uint8_t *const reg, uint8_t *const reg_count, uint8_t depth)
{
  uint8_t count = reg_count[bit];
  
  if(BIT_TST(reg_in, bit))
  {
    if(count < depth) { count++; }
    else              { BIT_SET(*reg, bit); }
  }
  else
  {
    if(count) { count--; }
    else      { BIT_CLR(*reg, bit); }
  }
  reg_count[bit] = count;
}

//==============================================================================
//    
//==============================================================================
#pragma inline = forced
static inline void filterKey(uint8_t key, uint8_t depth)
{
  if(key != 0xFF)
  {
    if(RegKeyCount < depth) { RegKeyCount++; }
    else                    { SET_KEY_DETECT; }
  }
  else
  {
    if(RegKeyCount) { RegKeyCount--; }
    else            { RESET_KEY_DETECT; }
  }
}

//==============================================================================
//   
//==============================================================================
#pragma vector = TIMER_1MS_VECTOR
__interrupt void RefreshLcdKbd(void)    //     
{
  CountInt++;
  
  if(!--Recount_1s)
  {
    SET_RECOUNT_1S;                     //   " "
    Recount_1s = _1000MS;
  }
  
  if(!--Recount_250ms)
  {
    SET_RECOUNT_250MS;
    Recount_250ms = _250MS;
  }

  if(!--BlinkCount)
  {
    if(EDIT && KEY_HOLD)  { RESET_BLINK; }
    else                  { TOGGLE_BLINK; }
    BlinkCount = BLINK_COUNT;
  }
  
  if(START_BEEP)                        //      
  {
    if(!--BeepCount)                    //    
    {
      BELL_OFF;                         //   
      RESET_START_BEEP;                 //      
    }
  }
  
//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
  {
    uint8_t key;                                        //   
//-----    ---------------------------------------------
    key = (~PINC) & 0x0F;
    key = TableKey[key];                                //   
    filterKey(key, KEY_DELAY_NOISE);                    //   
//-----    ----------------------------------------------
    if(!KEY_DETECT)                                     //  ?
    {
      RESET_KEY_HOLD;                                   //    
    }
    else
    {
      if(KEY_HOLD)                                      //  ?
      {                                                 // 
        if(KeyDelay == CountInt)                        //    /?
        {
          Key = key;                                    //   
          KeyDelay = CountInt + KEY_DELAY_HOLD;         //       
          SET_KEY_PRESS;
        }
      }
      else
      {                                                 //  
        BeepCount = DELAY_BEEP;                         //     
        BELL_ON;                                        //    
        SET_START_BEEP;                                 //      
        SET_KEY_HOLD;                                   //   
        Key = key;                                      //   
        KeyDelay = CountInt + KEY_DELAY_1PRESS;         //    
        SET_KEY_PRESS;
      }
    }
  }
  
//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
  {
    uint8_t reg_in;

    if(RegInBit)  { RegInBit--; }
    else          { RegInBit = 7; }
    
    REG_IN_1_ON;
    REG_DELAY;
    reg_in = REG_IN_PIN ^ REG_IN_MASK_1;   //   
    REG_IN_1_OFF;
    filterRegIn(RegInBit, reg_in, &RegIn_1, RegIn_1Count, REG_IN_DELAY_NOISE);
    
    REG_IN_2_ON;
    REG_DELAY;
    reg_in = REG_IN_PIN ^ REG_IN_MASK_2;   //   
    REG_IN_2_OFF;
    filterRegIn(RegInBit, reg_in, &RegIn_2, RegIn_2Count, REG_IN_DELAY_NOISE);
  }

  DDRC = 0xFF;
//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
  {
    REG_DATA_PORT = RegOut_1 ^ REG_OUT_MASK_1;   //   
    REG_DELAY;
    REG_OUT_1_ON;
    REG_DELAY;
    REG_OUT_1_OFF;
    
    REG_DELAY;

    REG_DATA_PORT = RegOut_2 ^ REG_OUT_MASK_2;   //   
    REG_DELAY;
    REG_OUT_2_ON;
    REG_DELAY;
    REG_OUT_2_OFF;
  }
  
//------------------------------------------------------------------------------
//   
//------------------------------------------------------------------------------
  {
    uint8_t cmd;
    uint8_t data1;
    uint8_t data2;
    uint8_t blink_data;
    uint8_t blink_count;
    
    if(DisplayCount > ((LCD_SIZE + LCD_ROW)/2))
      DisplayCount = 0;
    cmd = LCD_TableCfg[DisplayCount];
    
    if(BIT_TST(cmd, 7))
    {
      LCD_WRITE_CMD(cmd);
    }
    else
    {
      blink_count = cmd & 0x07;
      blink_data = DisplayBlink[cmd >> 3];
      
      if(BIT_TST(blink_data, blink_count) && BLINK)           //    
        data1 = 0x20;                                         //  
      else
      {
        data1 = DisplayOut[cmd];
//        data = LCD_TableCharDecode[data];                     //    LCD_TEXT_XX()
      }

      data2 = cmd + 80;
      blink_count = data2 & 0x07;
      blink_data = DisplayBlink[data2 >> 3];
      
      if(BIT_TST(blink_data, blink_count) && BLINK)           //    
        data2 = 0x20;                                         //  
      else
      {
        data2 = DisplayOut[data2];
//        data = LCD_TableCharDecode[data];                     //    LCD_TEXT_XX()
      }
      LCD_WRITE_DATA(data1, data2);
    }
    DisplayCount++;
  }
//-----      ---------------------------------------
  DDRC = 0x00;
//  PORTC = 0xFF;
  PORTC = 0x00;   //  Proteus
}
